@@ -1,11 +1,14 @@
 import functools
 import numpy as np
 import pandas as pd
+import re
 import warnings
 from collections import defaultdict
 from datetime import datetime
+from pandas.tslib import OutOfBoundsDatetime
 
 from .core import indexing, utils
+from .core.formatting import format_timestamp
 from .core.variable import as_variable, Variable
 from .core.pycompat import iteritems, bytes_type, unicode_type, OrderedDict
 
@@ -60,6 +63,39 @@ def mask_and_scale(array, fill_value=None, scale_factor=None, add_offset=None,
     return values
 
 
+def _netcdf_to_numpy_timeunit(units):
+    return {'seconds': 's', 'minutes': 'm', 'hours': 'h', 'days': 'D'}[units]
+
+
+def _unpack_netcdf_time_units(units):
+    matches = re.match('(\S+) since (.+)', units).groups()
+    if not matches:
+        raise ValueError('invalid time units: %s' % units)
+    delta, ref_date = matches
+    return delta, ref_date
+
+
+def _decode_netcdf_datetime(num_dates, units, calendar):
+    import netCDF4 as nc4
+
+    dates = np.asarray(nc4.num2date(num_dates, units, calendar))
+    if (dates[np.nanargmin(num_dates)].year < 1678
+            or dates[np.nanargmax(num_dates)].year >= 2262):
+        warnings.warn('Unable to decode time axis into full '
+                      'numpy.datetime64 objects, continuing using dummy '
+                      'netCDF4.datetime objects instead, reason: dates out'
+                      ' of range', RuntimeWarning, stacklevel=3)
+    else:
+        try:
+            dates = nctime_to_nptime(dates)
+        except ValueError as e:
+            warnings.warn('Unable to decode time axis into full '
+                          'numpy.datetime64 objects, continuing using '
+                          'dummy netCDF4.datetime objects instead, reason:'
+                          '{0}'.format(e), RuntimeWarning, stacklevel=3)
+    return dates
+
+
 def decode_cf_datetime(num_dates, units, calendar=None):
     """Given an array of numeric dates in netCDF format, convert it into a
     numpy array of date time objects.
@@ -72,98 +108,69 @@ def decode_cf_datetime(num_dates, units, calendar=None):
     --------
     netCDF4.num2date
     """
-    import netCDF4 as nc4
-    num_dates = np.asarray(num_dates).astype(float)
+    num_dates = np.asarray(num_dates, dtype=float)
+    flat_num_dates = num_dates.ravel()
+    orig_shape = num_dates.shape
     if calendar is None:
         calendar = 'standard'
 
-    def nan_safe_num2date(num):
-        return pd.NaT if np.isnan(num) else nc4.num2date(num, units, calendar)
+    delta, ref_date = _unpack_netcdf_time_units(units)
 
-    min_num = np.nanmin(num_dates)
-    max_num = np.nanmax(num_dates)
-    min_date = nan_safe_num2date(min_num)
-    if num_dates.size > 1:
-        max_date = nan_safe_num2date(max_num)
-    else:
-        max_date = min_date
+    try:
+        if calendar not in _STANDARD_CALENDARS:
+            raise OutOfBoundsDatetime
 
-    if ((calendar not in _STANDARD_CALENDARS
-            or min_date.year < 1678 or max_date.year >= 2262)
-            and min_date is not pd.NaT):
+        delta = _netcdf_to_numpy_timeunit(delta)
+        ref_date = pd.Timestamp(ref_date)
 
-        dates = nc4.num2date(num_dates, units, calendar)
+        dates = (pd.to_timedelta(num_dates.ravel(), delta) + ref_date).values
+    except OutOfBoundsDatetime:
+        dates = _decode_netcdf_datetime(flat_num_dates, units, calendar)
 
-        if min_date.year >= 1678 and max_date.year < 2262:
-            try:
-                dates = nctime_to_nptime(dates)
-            except ValueError as e:
-                warnings.warn('Unable to decode time axis into full '
-                              'numpy.datetime64 objects, continuing using '
-                              'dummy netCDF4.datetime objects instead, reason:'
-                              '{0}'.format(e), RuntimeWarning, stacklevel=2)
-                dates = np.asarray(dates)
-        else:
-            warnings.warn('Unable to decode time axis into full '
-                          'numpy.datetime64 objects, continuing using dummy '
-                          'netCDF4.datetime objects instead, reason: dates out'
-                          ' of range', RuntimeWarning, stacklevel=2)
-            dates = np.asarray(dates)
-
-    else:
-        # we can safely use np.datetime64 with nanosecond precision (pandas
-        # likes ns precision so it can directly make DatetimeIndex objects)
-        if pd.isnull(min_num):
-            # pandas.NaT doesn't cast to numpy.datetime64('NaT'), so handle it
-            # separately
-            dates = np.repeat(np.datetime64('NaT'), num_dates.size)
-        elif min_num == max_num:
-            # we can't safely divide by max_num - min_num
-            dates = np.repeat(np.datetime64(min_date), num_dates.size)
-            if dates.size > 1:
-                # don't bother with one element, since it will be fixed at
-                # min_date and isn't indexable anyways
-                dates[np.isnan(num_dates)] = np.datetime64('NaT')
-        else:
-            # Calculate the date as a np.datetime64 array from linear scaling
-            # of the max and min dates calculated via num2date.
-            flat_num_dates = num_dates.reshape(-1)
-            # Use second precision for the timedelta to decrease the chance of
-            # a numeric overflow
-            time_delta = np.timedelta64(max_date - min_date).astype('m8[s]')
-            if time_delta != max_date - min_date:
-                raise ValueError('unable to exactly represent max_date minus'
-                                 'min_date with second precision')
-            # apply the numerator and denominator separately so we don't need
-            # to cast to floating point numbers under the assumption that all
-            # dates can be given exactly with ns precision
-            numerator = flat_num_dates - min_num
-            denominator = max_num - min_num
-            dates = (time_delta * numerator / denominator
-                     + np.datetime64(min_date))
-        # restore original shape and ensure dates are given in ns
-        dates = dates.reshape(num_dates.shape).astype('M8[ns]')
-
-    return dates
+    return dates.reshape(num_dates.shape)
 
 
-def guess_time_units(dates):
-    """Given an array of dates suitable for input to `pandas.DatetimeIndex`,
-    returns a CF compatible time-unit string of the form "{time_unit} since
-    {date[0]}", where `time_unit` is 'days', 'hours', 'minutes' or 'seconds'
-    (the first one that can evenly divide all unique time deltas in `dates`)
+def decode_cf_timedelta(num_timedeltas, units):
+    """Given an array of numeric timedeltas in netCDF format, convert it into a
+    numpy timedelta64[ns] array.
     """
-    dates = pd.DatetimeIndex(np.asarray(dates).reshape(-1))
-    unique_timedeltas = np.unique(np.diff(dates.values))
+    units = _netcdf_to_numpy_timeunit(units)
+    return pd.to_timedelta(np.asarray(num_timedeltas), unit=units, box=False)
+
+
+TIME_UNITS = set(['days', 'hours', 'minutes', 'seconds'])
+
+def _infer_time_units_from_diff(unique_timedeltas):
     for time_unit, delta in [('days', 86400), ('hours', 3600),
                              ('minutes', 60), ('seconds', 1)]:
         unit_delta = np.timedelta64(10 ** 9 * delta, 'ns')
         diffs = unique_timedeltas / unit_delta
         if np.all(diffs == diffs.astype(int)):
-            break
-    else:
-        raise ValueError('could not automatically determine time units')
-    return '%s since %s' % (time_unit, dates[0])
+            return time_unit
+    raise ValueError('could not automatically determine time units')
+
+
+def infer_datetime_units(dates):
+    """Given an array of datetimes, returns a CF compatible time-unit string of
+    the form "{time_unit} since {date[0]}", where `time_unit` is 'days',
+    'hours', 'minutes' or 'seconds' (the first one that can evenly divide all
+    unique time deltas in `dates`)
+    """
+    dates = pd.to_datetime(np.asarray(dates), box=False)
+    unique_timedeltas = np.unique(np.diff(dates[pd.notnull(dates)]))
+    units = _infer_time_units_from_diff(unique_timedeltas)
+    return '%s since %s' % (units, pd.Timestamp(dates[0]))
+
+
+def infer_timedelta_units(deltas):
+    """Given an array of timedeltas, returns a CF compatible time-unit from
+    {'days', 'hours', 'minutes' 'seconds'} (the first one that can evenly
+    divide all unique time deltas in `deltas`)
+    """
+    deltas = pd.to_timedelta(np.asarray(deltas), box=False)
+    unique_timedeltas = np.unique(deltas[pd.notnull(deltas)])
+    units = _infer_time_units_from_diff(unique_timedeltas)
+    return units
 
 
 def nctime_to_nptime(times):
@@ -176,6 +183,17 @@ def nctime_to_nptime(times):
     return new
 
 
+def _cleanup_netcdf_time_units(units):
+    delta, ref_date = _unpack_netcdf_time_units(units)
+    try:
+        units = '%s since %s' % (delta, format_timestamp(ref_date))
+    except OutOfBoundsDatetime:
+        # don't worry about reifying the units if they're out of bounds
+        pass
+    return units
+
+
+
 def encode_cf_datetime(dates, units=None, calendar=None):
     """Given an array of datetime objects, returns the tuple `(num, units,
     calendar)` suitable for a CF complient time variable.
@@ -193,7 +211,10 @@ def encode_cf_datetime(dates, units=None, calendar=None):
     dates = np.asarray(dates)
 
     if units is None:
-        units = guess_time_units(dates)
+        units = infer_datetime_units(dates)
+    else:
+        units = _cleanup_netcdf_time_units(units)
+
     if calendar is None:
         calendar = 'proleptic_gregorian'
 
@@ -203,14 +224,29 @@ def encode_cf_datetime(dates, units=None, calendar=None):
         # note: numpy's broken datetime conversion only works for us precision
         dates = dates.astype('M8[us]').astype(datetime)
 
-    if hasattr(dates, 'ndim') and dates.ndim == 0:
-        # unpack dates because date2num doesn't like 0-dimensional arguments
-        dates = dates.item()
+    def encode_datetime(d):
+        return np.nan if d is None else nc4.date2num(d, units, calendar)
 
-    num = nc4.date2num(dates, units, calendar)
+    num = np.array([encode_datetime(d) for d in dates.flat])
+    num = num.reshape(dates.shape)
     return (num, units, calendar)
 
 
+def encode_cf_timedelta(timedeltas, units=None):
+    if units is None:
+        units = infer_timedelta_units(timedeltas)
+
+    np_unit = _netcdf_to_numpy_timeunit(units)
+    num = timedeltas.astype('timedelta64[%s]' % np_unit).view(np.int64)
+
+    missing = pd.isnull(timedeltas)
+    if np.any(missing):
+        num = num.astype(float)
+        num[missing] = np.nan
+
+    return (num, units)
+
+
 class MaskedAndScaledArray(utils.NDArrayMixin):
     """Wrapper around array-like objects to create a new indexable object where
     values, when accessesed, are automatically scaled and masked according to
@@ -288,6 +324,23 @@ class DecodedCFDatetimeArray(utils.NDArrayMixin):
                                   calendar=self.calendar)
 
 
+class DecodedCFTimedeltaArray(utils.NDArrayMixin):
+    """Wrapper around array-like objects to create a new indexable object where
+    values, when accessesed, are automatically converted into timedelta objects
+    using decode_cf_timedelta.
+    """
+    def __init__(self, array, units):
+        self.array = array
+        self.units = units
+
+    @property
+    def dtype(self):
+        return np.dtype('timedelta64[ns]')
+
+    def __getitem__(self, key):
+        return decode_cf_timedelta(self.array[key], units=self.units)
+
+
 class CharToStringArray(utils.NDArrayMixin):
     """Wrapper around array-like objects to create a new indexable object where
     values, when accessesed, are automatically concatenated along the last
@@ -339,25 +392,26 @@ class CharToStringArray(utils.NDArrayMixin):
 def string_to_char(arr):
     """Like netCDF4.stringtochar, but faster and more flexible.
     """
-    arr = np.asarray(arr)
+    # ensure the array is contiguous
+    arr = np.array(arr, copy=False, order='C')
     kind = arr.dtype.kind
     if kind not in ['U', 'S']:
         raise ValueError('argument must be a string')
-    return arr.view(kind + '1').reshape(*[arr.shape + (-1,)])
+    return arr.reshape(arr.shape + (1,)).view(kind + '1')
 
 
 def char_to_string(arr):
     """Like netCDF4.chartostring, but faster and more flexible.
     """
     # based on: http://stackoverflow.com/a/10984878/809705
-    arr = np.asarray(arr)
+    arr = np.array(arr, copy=False, order='C')
     kind = arr.dtype.kind
     if kind not in ['U', 'S']:
         raise ValueError('argument must be a string')
     return arr.view(kind + str(arr.shape[-1]))[..., 0]
 
 
-def _safe_setitem(dest, key, value):
+def safe_setitem(dest, key, value):
     if key in dest:
         raise ValueError('Failed hard to prevent overwriting key %r' % key)
     dest[key] = value
@@ -369,9 +423,9 @@ def pop_to(source, dest, key, default=None):
     None values are not passed on.  If k already exists in dest an
     error is raised.
     """
-    value = source.pop(key, default)
+    value = source.pop(key, None)
     if value is not None:
-        _safe_setitem(dest, key, value)
+        safe_setitem(dest, key, value)
     return value
 
 
@@ -383,16 +437,21 @@ def maybe_encode_datetime(var):
     if (np.issubdtype(var.dtype, np.datetime64)
             or (var.dtype.kind == 'O'
                 and isinstance(var.values.flat[0], datetime))):
-
         dims, values, attrs, encoding = _var_as_tuple(var)
-        if 'units' in attrs or 'calendar' in attrs:
-            raise ValueError(
-                "Failed hard to prevent overwriting 'units' or 'calendar'")
-
         (values, units, calendar) = encode_cf_datetime(
             values, encoding.pop('units', None), encoding.pop('calendar', None))
-        attrs['units'] = units
-        attrs['calendar'] = calendar
+        safe_setitem(attrs, 'units', units)
+        safe_setitem(attrs, 'calendar', calendar)
+        var = Variable(dims, values, attrs, encoding)
+    return var
+
+
+def maybe_encode_timedelta(var):
+    if np.issubdtype(var.dtype, np.timedelta64):
+        dims, values, attrs, encoding = _var_as_tuple(var)
+        values, units = encode_cf_timedelta(
+            values, encoding.pop('units', None))
+        safe_setitem(attrs, 'units', units)
         var = Variable(dims, values, attrs, encoding)
     return var
 
@@ -451,7 +510,7 @@ def _infer_dtype(array):
     else:
         dtype = np.array(array.flat[0]).dtype
         if dtype.kind in ['S', 'U']:
-            # don't just use inferred_dtype to avoid truncating arrays to
+            # don't just use inferred dtype to avoid truncating arrays to
             # the length of their first element
             dtype = np.dtype(dtype.kind)
         elif dtype.kind == 'O':
@@ -510,6 +569,7 @@ def encode_cf_variable(var, needs_copy=True):
         A variable which has been encoded as described above.
     """
     var = maybe_encode_datetime(var)
+    var = maybe_encode_timedelta(var)
     var, needs_copy = maybe_encode_offset_and_scale(var, needs_copy)
     var, needs_copy = maybe_encode_fill_value(var, needs_copy)
     var = maybe_encode_dtype(var, needs_copy)
@@ -584,18 +644,24 @@ def decode_cf_variable(var, concat_characters=True, mask_and_scale=True,
             data = MaskedAndScaledArray(data, fill_value, scale_factor,
                                         add_offset, dtype)
 
-    if decode_times:
-        if 'units' in attributes and 'since' in attributes['units']:
+    if decode_times and 'units' in attributes:
+        if 'since' in attributes['units']:
+            # datetime
             units = pop_to(attributes, encoding, 'units')
             calendar = pop_to(attributes, encoding, 'calendar')
             data = DecodedCFDatetimeArray(data, units, calendar)
+        elif attributes['units'] in TIME_UNITS:
+            # timedelta
+            units = pop_to(attributes, encoding, 'units')
+            data = DecodedCFTimedeltaArray(data, units)
 
     return Variable(dimensions, indexing.LazilyIndexedArray(data),
                     attributes, encoding=encoding)
 
 
-def decode_cf_variables(variables, concat_characters=True, mask_and_scale=True,
-                        decode_times=True):
+def decode_cf_variables(variables, attributes, concat_characters=True,
+                        mask_and_scale=True, decode_times=True,
+                        decode_coords=True):
     """
     Decode a several CF encoded variables.
 
@@ -615,6 +681,8 @@ def decode_cf_variables(variables, concat_characters=True, mask_and_scale=True,
                 return False
         return True
 
+    coord_names = set()
+
     new_vars = OrderedDict()
     for k, v in iteritems(variables):
         concat = (concat_characters and v.dtype.kind == 'S' and v.ndim > 0 and
@@ -622,7 +690,66 @@ def decode_cf_variables(variables, concat_characters=True, mask_and_scale=True,
         new_vars[k] = decode_cf_variable(
             v, concat_characters=concat, mask_and_scale=mask_and_scale,
             decode_times=decode_times)
-    return new_vars
+        if decode_coords:
+            coordinates = new_vars[k].attrs.pop('coordinates', None)
+            if coordinates is not None:
+                coord_names.update(coordinates.split())
+
+    if decode_coords and 'coordinates' in attributes:
+        attributes = OrderedDict(attributes)
+        coord_names.update(attributes.pop('coordinates').split())
+
+    return new_vars, attributes, coord_names
+
+
+def decode_cf(obj, concat_characters=True, mask_and_scale=True,
+              decode_times=True, decode_coords=True):
+    """Decode the given Dataset or Datastore according to CF conventions into
+    a new Dataset.
+
+    Parameters
+    ----------
+    obj : Dataset or DataStore
+        Object to decode.
+    concat_characters : bool, optional
+        Should character arrays be concatenated to strings, for
+        example: ['h', 'e', 'l', 'l', 'o'] -> 'hello'
+    mask_and_scale: bool, optional
+        Lazily scale (using scale_factor and add_offset) and mask
+        (using _FillValue).
+    decode_times : bool, optional
+        Decode cf times (e.g., integers since 'hours since 2000-01-01') to
+        np.datetime64.
+    decode_coords : bool, optional
+        Use the 'coordinates' attribute on variable (or the dataset itself) to
+        identify coordinates.
+
+    Returns
+    -------
+    decoded : Dataset
+    """
+    from .core.dataset import Dataset
+    from .backends.common import AbstractDataStore
+
+    if isinstance(obj, Dataset):
+        vars = obj._arrays
+        attrs = obj.attrs
+        extra_coords = set(obj.coords)
+        file_obj = obj._file_obj
+    elif isinstance(obj, AbstractDataStore):
+        vars, attrs = obj.load()
+        extra_coords = set()
+        file_obj = obj
+    else:
+        raise TypeError('can only decode Dataset or DataStore objects')
+
+    vars, attrs, coord_names = decode_cf_variables(
+        vars, attrs, concat_characters, mask_and_scale, decode_times,
+        decode_coords)
+    ds = Dataset(vars, attrs=attrs)
+    ds = ds.set_coords(coord_names.union(extra_coords))
+    ds._file_obj = file_obj
+    return ds
 
 
 def cf_decoder(variables, attributes,
@@ -647,20 +774,79 @@ def cf_decoder(variables, attributes,
         (using _FillValue).
     decode_times : bool
         Decode cf times ('hours since 2000-01-01') to np.datetime64.
-    decode_cf : bool
-        If false this skips decoding.  This is around for backward
-        compatibility.
 
     Returns
     -------
     decoded_variables : dict
-        A dictionary mapping from variable name to xray.Variable,
+        A dictionary mapping from variable name to xray.Variable objects.
     decoded_attributes : dict
-        A dictionary mapping from attribute name to value
+        A dictionary mapping from attribute name to values.
     """
-    new_vars = decode_cf_variables(variables, concat_characters,
-                                   mask_and_scale, decode_times)
-    return new_vars, attributes
+    variables, attributes, _ = decode_cf_variables(
+        variables, attributes, concat_characters, mask_and_scale, decode_times)
+    return variables, attributes
+
+
+def _encode_coordinates(variables, attributes, non_dim_coord_names):
+    # calculate global and variable specific coordinates
+    non_dim_coord_names = set(non_dim_coord_names)
+    global_coordinates = non_dim_coord_names.copy()
+    variable_coordinates = defaultdict(set)
+    for coord_name in non_dim_coord_names:
+        target_dims = variables[coord_name].dims
+        for k, v in variables.items():
+            if (k not in non_dim_coord_names and k not in v.dims
+                    and any(d in target_dims for d in v.dims)):
+                variable_coordinates[k].add(coord_name)
+                global_coordinates.discard(coord_name)
+
+    variables = OrderedDict((k, v.copy(deep=False))
+                            for k, v in variables.items())
+
+    # These coordinates are saved according to CF conventions
+    for var_name, coord_names in variable_coordinates.items():
+        attrs = variables[var_name].attrs
+        if 'coordinates' in attrs:
+            raise ValueError('cannot serialize coordinates because variable '
+                             "%s already has an attribute 'coordinates'"
+                             % var_name)
+        attrs['coordinates'] = ' '.join(map(str, coord_names))
+
+    # These coordinates are not associated with any particular variables, so we
+    # save them under a global 'coordinates' attribute so xray can roundtrip
+    # the dataset faithfully. Because this serialization goes beyond CF
+    # conventions, only do it if necessary.
+    # Reference discussion:
+    # http://mailman.cgd.ucar.edu/pipermail/cf-metadata/2014/057771.html
+    if global_coordinates:
+        attributes = OrderedDict(attributes)
+        if 'coordinates' in attributes:
+            raise ValueError('cannot serialize coordinates because the global '
+                             "attribute 'coordinates' already exists")
+        attributes['coordinates'] = ' '.join(map(str, global_coordinates))
+
+    return variables, attributes
+
+
+def encode_dataset_coordinates(dataset):
+    """Encode coordinates on the given dataset object into variable specific
+    and global attributes.
+
+    When possible, this is done according to CF conventions.
+
+    Parameters
+    ----------
+    dataset : Dataset
+        Object to encode.
+
+    Returns
+    -------
+    variables : dict
+    attrs : dict
+    """
+    non_dim_coord_names = set(dataset.coords) - set(dataset.dims)
+    return _encode_coordinates(dataset._arrays, dataset.attrs,
+                               non_dim_coord_names=non_dim_coord_names)
 
 
 def cf_encoder(variables, attributes):
