@@ -6,6 +6,7 @@ from collections import defaultdict
 import numpy as np
 
 from . import utils
+from .common import _maybe_promote
 from .pycompat import iteritems, OrderedDict
 from .variable import as_variable, Variable, Coordinate, broadcast_variables
 
@@ -38,7 +39,7 @@ def align(*objects, **kwargs):
         Method for joining the indexes of the passed objects along each
         dimension:
          - 'outer': use the union of object indexes
-         - 'outer': use the intersection of object indexes
+         - 'inner': use the intersection of object indexes
          - 'left': use indexes from the first object with each dimension
          - 'right': use indexes from the last object with each dimension
     copy : bool, optional
@@ -133,42 +134,27 @@ def reindex_variables(variables, indexes, indexers, copy=True):
     def var_indexers(var, indexers):
         return tuple(indexers.get(d, slice(None)) for d in var.dims)
 
-    def get_fill_value_and_dtype(dtype):
-        # N.B. these casting rules should match pandas
-        if np.issubdtype(dtype, np.datetime64):
-            fill_value = np.datetime64('NaT')
-        elif any(np.issubdtype(dtype, t) for t in (int, float)):
-            # convert to floating point so NaN is valid
-            dtype = float
-            fill_value = np.nan
-        else:
-            dtype = object
-            fill_value = np.nan
-        return fill_value, dtype
-
     # create variables for the new dataset
     reindexed = OrderedDict()
     for name, var in iteritems(variables):
         if name in indexers:
-            new_var = indexers[name]
-            if not hasattr(new_var, 'dims') or not hasattr(new_var, 'values'):
-                new_var = Coordinate(var.dims, new_var, var.attrs,
-                                     var.encoding)
-            elif copy:
-                new_var = as_variable(new_var).copy()
+            # no need to copy, because index data is immutable
+            new_var = Coordinate(var.dims, indexers[name], var.attrs,
+                                 var.encoding)
         else:
             assign_to = var_indexers(var, to_indexers)
             assign_from = var_indexers(var, from_indexers)
 
             if any_not_full_slices(assign_to):
                 # there are missing values to in-fill
-                fill_value, dtype = get_fill_value_and_dtype(var.dtype)
+                dtype, fill_value = _maybe_promote(var.dtype)
                 shape = tuple(length if is_full_slice(idx) else idx.size
                               for idx, length in zip(assign_to, var.shape))
                 data = np.empty(shape, dtype=dtype)
                 data[:] = fill_value
                 # create a new Variable so we can use orthogonal indexing
-                new_var = Variable(var.dims, data, var.attrs)
+                # use fastpath=True to avoid dtype inference
+                new_var = Variable(var.dims, data, var.attrs, fastpath=True)
                 new_var[assign_to] = var[assign_from].values
             elif any_not_full_slices(assign_from):
                 # type coercion is not necessary as there are no missing
