@@ -9,7 +9,7 @@ from . import groupby
 from . import ops
 from . import utils
 from . import variable
-from .common import AbstractArray
+from .common import AbstractArray, AttrAccessMixin
 from .coordinates import DataArrayCoordinates, Indexes
 from .dataset import Dataset
 from .pycompat import iteritems, basestring, OrderedDict, zip
@@ -21,7 +21,7 @@ def _infer_coords_and_dims(shape, coords, dims):
     """All the logic for creating a new DataArray"""
 
     if (coords is not None and not utils.is_dict_like(coords)
-            and not len(coords) == len(shape)):
+            and len(coords) != len(shape)):
         raise ValueError('coords is not dict-like, but it has %s items, '
                          'which does not match the %s dimensions of the '
                          'data' % (len(coords), len(shape)))
@@ -62,12 +62,20 @@ class _LocIndexer(object):
         self.data_array = data_array
 
     def _remap_key(self, key):
-        label_indexers = self.data_array._key_to_indexers(key)
-        indexers = []
-        for dim, label in iteritems(label_indexers):
+        def lookup_positions(dim, labels):
             index = self.data_array.indexes[dim]
-            indexers.append(indexing.convert_label_indexer(index, label))
-        return tuple(indexers)
+            return indexing.convert_label_indexer(index, labels)
+
+        if utils.is_dict_like(key):
+            return dict((dim, lookup_positions(dim, labels))
+                        for dim, labels in iteritems(key))
+        else:
+            if not isinstance(key, tuple):
+                key = (key,)
+            # note: it's OK if there are fewer keys than dimensions: zip will
+            # finish early in that case (we don't need to insert colons)
+            return tuple(lookup_positions(dim, labels) for dim, labels
+                         in zip(self.data_array.dims, key))
 
     def __getitem__(self, key):
         return self.data_array[self._remap_key(key)]
@@ -76,11 +84,7 @@ class _LocIndexer(object):
         self.data_array[self._remap_key(key)] = value
 
 
-# singleton object for checking
-UNNAMED = object()
-
-
-class DataArray(AbstractArray):
+class DataArray(AbstractArray, AttrAccessMixin):
     """N-dimensional array with labeled coordinates and dimensions.
 
     DataArray provides a wrapper around numpy ndarrays that uses labeled
@@ -118,17 +122,16 @@ class DataArray(AbstractArray):
     attrs : OrderedDict
         Dictionary for holding arbitrary metadata.
     """
-    def __init__(self, data=None, coords=None, dims=None, name=None,
+    def __init__(self, data, coords=None, dims=None, name=None,
                  attrs=None, encoding=None):
         """
         Parameters
         ----------
-        data : array_like, optional
+        data : array_like
             Values for this array. Must be a ``numpy.ndarray``, ndarray like,
             or castable to an ``ndarray``. If a self-described xray or pandas
             object, attempst are made to use this array's metadata to fill in
-            other unspecified arguments. This argument is required unless the
-            'dataset' argument is provided.
+            other unspecified arguments.
         coords : sequence or dict of array_like objects, optional
             Coordinates (tick labels) to use for indexing along each dimension.
             If dict-like, should be a mapping from dimension names to the
@@ -330,16 +333,19 @@ class DataArray(AbstractArray):
         utils.alias_warning('dimensions', 'dims')
         return self.dims
 
-    def _key_to_indexers(self, key):
-        return OrderedDict(
-            zip(self.dims, indexing.expanded_indexer(key, self.ndim)))
+    def _item_key_to_dict(self, key):
+        if utils.is_dict_like(key):
+            return key
+        else:
+            key = indexing.expanded_indexer(key, self.ndim)
+            return dict(zip(self.dims, key))
 
     def __getitem__(self, key):
         if isinstance(key, basestring):
             return self.coords[key]
         else:
             # orthogonal array indexing
-            return self.isel(**self._key_to_indexers(key))
+            return self.isel(**self._item_key_to_dict(key))
 
     def __setitem__(self, key, value):
         if isinstance(key, basestring):
@@ -351,12 +357,17 @@ class DataArray(AbstractArray):
     def __delitem__(self, key):
         del self._dataset[key]
 
+    @property
+    def __attr_sources__(self):
+        """List of places to look-up items for attribute-style access"""
+        return [self.coords, self.attrs]
+
     def __contains__(self, key):
         return key in self._dataset
 
     @property
     def loc(self):
-        """Attribute for location based indexing like pandas..
+        """Attribute for location based indexing like pandas.
         """
         return _LocIndexer(self)
 
@@ -841,6 +852,19 @@ class DataArray(AbstractArray):
         ds = Dataset.from_dataframe(df)
         return cls._new_from_dataset_no_copy(ds, series.name)
 
+    def to_cdms2(self):
+        """Convert this array into a cdms2.Variable
+        """
+        from ..convert import to_cdms2
+        return to_cdms2(self)
+
+    @classmethod
+    def from_cdms2(cls, variable):
+        """Convert a cdms2.Variable into an xray.DataArray
+        """
+        from ..convert import from_cdms2
+        return from_cdms2(variable)
+
     def _all_compat(self, other, compat_str):
         """Helper function for equals and identical"""
         compat = lambda x, y: getattr(x.variable, compat_str)(y.variable)
@@ -880,6 +904,8 @@ class DataArray(AbstractArray):
         except (TypeError, AttributeError):
             return False
 
+    __default_name = object()
+
     def _result_name(self, other=None):
 
         if self.name in self.dims:
@@ -891,7 +917,7 @@ class DataArray(AbstractArray):
             # shortcut
             return self.name
 
-        other_name = getattr(other, 'name', UNNAMED)
+        other_name = getattr(other, 'name', self.__default_name)
         other_dims = getattr(other, 'dims', ())
 
         if other_name in other_dims:
@@ -900,7 +926,7 @@ class DataArray(AbstractArray):
 
         # use the same naming heuristics as pandas:
         # https://github.com/ContinuumIO/blaze/issues/458#issuecomment-51936356
-        if other_name is UNNAMED or other_name == self.name:
+        if other_name is self.__default_name or other_name == self.name:
             return self.name
 
         return None
