@@ -18,21 +18,24 @@ from . import alignment
 from . import formatting
 from .. import backends, conventions
 from .coordinates import DatasetCoordinates, Indexes
-from .utils import (Frozen, SortedKeysDict, ChainMap,
-                    multi_index_from_product)
+from .common import ImplementsDatasetReduce, AttrAccessMixin
+from .utils import Frozen, SortedKeysDict, ChainMap
 from .pycompat import iteritems, itervalues, basestring, OrderedDict
 
 
-def open_dataset(nc, decode_cf=True, mask_and_scale=True, decode_times=True,
-                 concat_characters=True, *args, **kwargs):
-    """Load a dataset from a file or file-like object.
+def open_dataset(filename_or_obj, decode_cf=True, mask_and_scale=True,
+                 decode_times=True, concat_characters=True, decode_coords=True,
+                 group=None):
+    """Load and decode a dataset from a file or file-like object.
 
     Parameters
     ----------
-    nc : str or file
-        Path to a netCDF4 file or an OpenDAP URL (opened with python-netCDF4)
-        or a file object or string serialization of a netCDF3 file (opened with
-        scipy.io.netcdf). If the filename ends with .gz, the file is gunzipped
+    filename_or_obj : str or file
+        Strings are intrepreted as a path to a netCDF file or an OpenDAP URL
+        and opened with python-netCDF4, unless the filename ends with .gz, in
+        which case the file is gunzipped and opened with scipy.io.netcdf (only
+        netCDF3 supported). File-like objects are opened with scipy.io.netcdf
+        (only netCDF3 supported).
     decode_cf : bool, optional
         Whether to decode these variables, assuming they were saved according
         to CF conventions.
@@ -49,49 +52,45 @@ def open_dataset(nc, decode_cf=True, mask_and_scale=True, decode_times=True,
         form string arrays. Dimensions will only be concatenated over (and
         removed) if they have no corresponding variable and if they are only
         used as the last dimension of character arrays.
-    *args, **kwargs : optional
-        Format specific loading options passed on to the datastore.
+    decode_coords : bool, optional
+        If True, decode the 'coordinates' attribute to identify coordinates in
+        the resulting dataset.
+    group : str, optional
+        NetCDF4 group in the given file to open (only works for netCDF4).
 
     Returns
     -------
     dataset : Dataset
         The newly created dataset.
     """
-    # move this to a classmethod Dataset.open?
-    # TODO: this check has the unfortunate side-effect that
-    # paths to files cannot start with 'CDF'.
-    if isinstance(nc, basestring):
-        # If the initialization nc is a string and
-        if nc.endswith('.gz'):
+    if isinstance(filename_or_obj, basestring):
+        if filename_or_obj.endswith('.gz'):
             # if the string ends with .gz, then gunzip and open as netcdf file
             if sys.version_info[:2] < (2, 7):
                 raise ValueError('reading a gzipped netCDF not '
                                  'supported on Python 2.6')
             try:
-                store = backends.ScipyDataStore(gzip.open(nc), *args, **kwargs)
+                store = backends.ScipyDataStore(gzip.open(filename_or_obj))
             except TypeError as e:
                 # TODO: gzipped loading only works with NetCDF3 files.
                 if 'is not a valid NetCDF 3 file' in e.message:
-                    raise ValueError("xray: gzipped file loading only supports NetCDF 3 files.")
+                    raise ValueError('gzipped file loading only supports '
+                                     'NetCDF 3 files.')
                 else:
-                    raise e
-        elif not nc.startswith('CDF'):
-            # nc does not appear to be a string holding the contents of a
-            # netcdf file so we treat it as a path and load it using the
-            # netCDF4 package
-            store = backends.NetCDF4DataStore(nc, *args, **kwargs)
+                    raise
+        else:
+            store = backends.NetCDF4DataStore(filename_or_obj, group=group)
     else:
-        # If nc is a file-like object we read it using
-        # the scipy.io.netcdf package
-        store = backends.ScipyDataStore(nc, *args, **kwargs)
+        # assume filename_or_obj is a file-like object
+        store = backends.ScipyDataStore(filename_or_obj)
+
     if decode_cf:
-        decoder = functools.partial(conventions.cf_decoder,
-                                    mask_and_scale=mask_and_scale,
-                                    decode_times=decode_times,
-                                    concat_characters=concat_characters)
+        return conventions.decode_cf(
+            store, mask_and_scale=mask_and_scale,
+            decode_times=decode_times, concat_characters=concat_characters,
+            decode_coords=decode_coords)
     else:
-        decoder = None
-    return Dataset.load_store(store, decoder=decoder)
+        return Dataset.load_store(store)
 
 
 # list of attributes of pd.DatetimeIndex that are ndarrays of time info
@@ -293,7 +292,17 @@ class Variables(Mapping):
         return formatting.vars_repr(self)
 
 
-class Dataset(Mapping, common.ImplementsDatasetReduce):
+class _LocIndexer(object):
+    def __init__(self, dataset):
+        self.dataset = dataset
+
+    def __getitem__(self, key):
+        if not utils.is_dict_like(key):
+            raise TypeError('can only lookup dictionaries from Dataset.loc')
+        return self.dataset.sel(**key)
+
+
+class Dataset(Mapping, ImplementsDatasetReduce, AttrAccessMixin):
     """A multi-dimensional, in memory, array database.
 
     A dataset resembles an in-memory representation of a NetCDF file, and
@@ -306,6 +315,11 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
     One dimensional variables with name equal to their dimension are index
     coordinates used for label based indexing.
     """
+    # class properties defined for the benefit of __setstate__, which otherwise
+    # runs into trouble because we overrode __getattr__
+    _attrs = None
+    _arrays = Frozen({})
+
     def __init__(self, variables=None, coords=None, attrs=None):
         """To load data from a file or file-like object, use the `open_dataset`
         function.
@@ -455,6 +469,9 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         utils.alias_warning('attributes', 'attrs', 3)
         self.attrs = value
 
+    def _attrs_copy(self):
+        return None if self._attrs is None else OrderedDict(self._attrs)
+
     @property
     def attrs(self):
         """Dictionary of global attributes on this dataset
@@ -508,6 +525,36 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         obj._file_obj = file_obj
         return obj
 
+    __default_attrs = object()
+
+    def _replace_vars_and_dims(self, variables, coord_names=None,
+                               attrs=__default_attrs):
+        """Fastpath constructor for internal use.
+
+        Preserves coord names and attributes; dimensions are recalculated from
+        the supplied variables.
+
+        The arguments are *not* copied when placed on the new dataset. It is up
+        to the caller to ensure that they have the right type and are not used
+        elsewhere.
+
+        Parameters
+        ----------
+        variables : OrderedDict
+        coord_names : set or None, optional
+        attrs : OrderedDict or None, optional
+
+        Returns
+        -------
+        new : Dataset
+        """
+        dims = _calculate_dims(variables)
+        if coord_names is None:
+            coord_names = self._coord_names.copy()
+        if attrs is self.__default_attrs:
+            attrs = self._attrs_copy()
+        return self._construct_direct(variables, coord_names, dims, attrs)
+
     def copy(self, deep=False):
         """Returns a copy of this dataset.
 
@@ -521,9 +568,8 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         else:
             variables = self._arrays.copy()
         # skip __init__ to avoid costly validation
-        attrs = None if self._attrs is None else self._attrs.copy()
         return self._construct_direct(variables, self._coord_names.copy(),
-                                      self._dims.copy(), attrs)
+                                      self._dims.copy(), self._attrs_copy())
 
     def _copy_listed(self, names, keep_attrs=True):
         """Create a new Dataset with the listed variables from this dataset and
@@ -575,6 +621,13 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
     def __iter__(self):
         return iter(self._arrays)
 
+    @property
+    def loc(self):
+        """Attribute for location based indexing. Only supports __getitem__,
+        and only when the key is a dict of the form {dim: labels}.
+        """
+        return _LocIndexer(self)
+
     @property
     def virtual_variables(self):
         """A frozenset of names that don't exist in this dataset but for which
@@ -602,6 +655,9 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         """
         from .dataarray import DataArray
 
+        if utils.is_dict_like(key):
+            return self.isel(**key)
+
         key = np.asarray(key)
         if key.ndim == 0:
             return DataArray._new_from_dataset(self, key.item())
@@ -619,6 +675,9 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         ``(dims, data[, attrs])``), add it to this dataset as a new
         variable.
         """
+        if utils.is_dict_like(key):
+            raise NotImplementedError('cannot yet use a dictionary as a key '
+                                      'to set Dataset values')
         self.merge({key: value}, inplace=True, overwrite_vars=[key])
 
     def __delitem__(self, key):
@@ -789,10 +848,10 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
 
     def dump_to_store(self, store, encoder=None):
         """Store dataset contents to a backends.*DataStore object."""
-        variables, attributes = self, self.attrs
+        variables, attrs = conventions.encode_dataset_coordinates(self)
         if encoder:
-            variables, attributes = encoder(variables, attributes)
-        store.store(variables, attributes)
+            variables, attrs = encoder(variables, attrs)
+        store.store(variables, attrs)
         store.sync()
 
     def to_netcdf(self, filepath, **kwdargs):
@@ -859,9 +918,7 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         for name, var in iteritems(self._arrays):
             var_indexers = dict((k, v) for k, v in indexers if k in var.dims)
             variables[name] = var.isel(**var_indexers)
-        obj = type(self)(variables, attrs=self.attrs)
-        obj._coord_names.update(self.coords)
-        return obj
+        return self._replace_vars_and_dims(variables)
 
     indexed = utils.function_alias(isel, 'indexed')
 
@@ -938,21 +995,23 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         """
         return self.reindex(copy=copy, **other.indexes)
 
-    def reindex(self, copy=True, **indexers):
+    def reindex(self, indexers=None, copy=True, **kw_indexers):
         """Conform this object onto a new set of indexes, filling in
         missing values with NaN.
 
         Parameters
         ----------
-        copy : bool, optional
-            If `copy=True`, the returned dataset contains only copied
-            variables. If `copy=False` and no reindexing is required then
-            original variables from this dataset are returned.
-        **indexers : dict
+        indexers : dict. optional
             Dictionary with keys given by dimension names and values given by
             arrays of coordinates tick labels. Any mis-matched coordinate values
             will be filled in with NaN, and any mis-matched dimension names will
             simply be ignored.
+        copy : bool, optional
+            If `copy=True`, the returned dataset contains only copied
+            variables. If `copy=False` and no reindexing is required then
+            original variables from this dataset are returned.
+        **kw_indexers : optional
+            Keyword arguments in the same form as ``indexers``.
 
         Returns
         -------
@@ -964,15 +1023,15 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         Dataset.reindex_like
         align
         """
+        indexers = utils.combine_pos_and_kw_args(indexers, kw_indexers,
+                                                 'reindex')
         if not indexers:
             # shortcut
             return self.copy(deep=True) if copy else self
 
         variables = alignment.reindex_variables(
             self._arrays, self.indexes, indexers, copy=copy)
-        obj = type(self)(variables, attrs=self.attrs)
-        obj._coord_names.update(self.coords)
-        return obj
+        return self._replace_vars_and_dims(variables)
 
     def rename(self, name_dict, inplace=False):
         """Returns a new object with renamed variables and dimensions.
@@ -1009,10 +1068,10 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         if inplace:
             self._dims = _calculate_dims(variables)
             self._arrays = variables
+            self._coord_names = coord_names
             obj = self
         else:
-            obj = type(self)(variables, attrs=self.attrs)
-        obj._coord_names = coord_names
+            obj = self._replace_vars_and_dims(variables, coord_names)
         return obj
 
     def update(self, other, inplace=True):
@@ -1148,9 +1207,7 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         variables = OrderedDict((k, v) for k, v in iteritems(self._arrays)
                                 if k not in drop)
         coord_names = set(k for k in self._coord_names if k in variables)
-        obj = type(self)(variables, attrs=self.attrs)
-        obj._coord_names = coord_names
-        return obj
+        return self._replace_vars_and_dims(variables, coord_names)
 
     unselect = utils.function_alias(drop_vars, 'unselect')
 
@@ -1365,10 +1422,8 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
                 variables[name] = var
 
         coord_names = set(k for k in self.coords if k in variables)
-        attrs = self.attrs if keep_attrs else {}
-        obj = type(self)(variables, attrs=attrs)
-        obj._coord_names = coord_names
-        return obj
+        attrs = self.attrs if keep_attrs else None
+        return self._replace_vars_and_dims(variables, coord_names, attrs)
 
     def apply(self, func, keep_attrs=False, args=(), **kwargs):
         """Apply a function over the variables in this dataset.
@@ -1505,10 +1560,8 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
 
     def _to_dataframe(self, ordered_dims):
         columns = [k for k in self if k not in self.dims]
-        orig_arrays = (self._arrays[k] for k in columns)
-        broadcast_arrays = variable.broadcast_variables(*orig_arrays)
-        data = [arr.transpose(*ordered_dims).values.reshape(-1)
-                for arr in broadcast_arrays]
+        data = [self._arrays[k].set_dims(ordered_dims).values.reshape(-1)
+                for k in columns]
         index = self.coords.to_index(ordered_dims)
         return pd.DataFrame(OrderedDict(zip(columns, data)), index=index)
 
@@ -1543,7 +1596,7 @@ class Dataset(Mapping, common.ImplementsDatasetReduce):
         if hasattr(idx, 'levels'):
             # it's a multi-index
             # expand the DataFrame to include the product of all levels
-            full_idx = multi_index_from_product(idx.levels, idx.names)
+            full_idx = pd.MultiIndex.from_product(idx.levels, names=idx.names)
             dataframe = dataframe.reindex(full_idx)
             dims = [name if name is not None else 'level_%i' % n
                     for n, name in enumerate(idx.names)]
