@@ -44,30 +44,35 @@ def allclose_or_equiv(arr1, arr2, rtol=1e-5, atol=1e-8):
     return np.isclose(arr1, arr2, rtol=rtol, atol=atol, equal_nan=True).all()
 
 
-def isnull(obj):
-    """Like pd.isnull, but with a bug-fix for 0d object arrays.
-    """
-    obj = np.asarray(obj)
-    if obj.ndim == 0:
-        # work around for pd.isnull not working for 0-dimensional object
-        # arrays: https://github.com/pydata/pandas/pull/7176 (should be fixed
-        # in pandas 0.14)
-        # use .item() instead of keeping around 0-dimensional arrays to avoid
-        # the numpy quirk where object arrays are checked as equal by identity
-        # (hence NaN in an object array is equal to itself):
-        obj = obj.item()
-        return obj != obj
-    else:
-        return pd.isnull(obj)
-
-
 def array_equiv(arr1, arr2):
     """Like np.array_equal, but also allows values to be NaN in both arrays
     """
     arr1, arr2 = np.asarray(arr1), np.asarray(arr2)
     if arr1.shape != arr2.shape:
         return False
-    return ((arr1 == arr2) | (isnull(arr1) & isnull(arr2))).all()
+    return ((arr1 == arr2) | (pd.isnull(arr1) & pd.isnull(arr2))).all()
+
+
+def as_shape(array, shape):
+    """Expand a numpy.ndarray to a new shape according to broadcasting rules
+    """
+    array = np.asarray(array)
+    if len(shape) < array.ndim:
+        raise ValueError('new shape must be greater than or equal to the '
+                         'number of array dimensions')
+
+    # based on numpy.lib.stride_tricks.broadcast_arrays
+    strides = [0] * (len(shape) - array.ndim) + list(array.strides)
+    for n, (new_size, old_size) in enumerate(zip(shape[-array.ndim:],
+                                                 array.shape)):
+        if new_size != old_size:
+            axis = array.ndim + n
+            if old_size != 1:
+                raise ValueError('shape mismatch: new shape differs from the '
+                                 'original shape on axis %r, but the original '
+                                 'shape is not 1 on that axis' % axis)
+            strides[axis] = 0
+    return np.lib.stride_tricks.as_strided(array, shape=shape, strides=strides)
 
 
 def safe_cast_to_index(array):
@@ -85,22 +90,12 @@ def safe_cast_to_index(array):
         index = array.to_index()
     else:
         kwargs = {}
-        if hasattr(array, 'dtype'):
-            if array.dtype == object or array.dtype.kind == 'm':
-                kwargs['dtype'] = object
+        if hasattr(array, 'dtype') and array.dtype.kind == 'O':
+            kwargs['dtype'] = object
         index = pd.Index(np.asarray(array), **kwargs)
     return index
 
 
-def multi_index_from_product(iterables, names=None):
-    """Like pandas.MultiIndex.from_product, but with a bug fix.
-    """
-    # fixed in 0.14: https://github.com/pydata/pandas/issues/6439
-    # note: pd.MultiIndex.from_product is new in pandas-0.13.1
-    coords = [np.asarray(v) for v in iterables]
-    return pd.MultiIndex.from_product(coords, names=names)
-
-
 def equivalent(first, second):
     """Compare two objects for equivalence (identity or equality), using
     array_equiv if either object is an ndarray
@@ -156,9 +151,10 @@ def remove_incompatible_items(first_dict, second_dict, compat=equivalent):
         Binary operator to determine if two values are compatible. By default,
         checks for equivalence.
     """
-    for k, v in iteritems(first_dict):
+    for k in list(first_dict):
         if (k not in second_dict
-                or (k in second_dict and not compat(v, second_dict[k]))):
+                or (k in second_dict and
+                    not compat(first_dict[k], second_dict[k]))):
             del first_dict[k]
 
 
@@ -166,6 +162,19 @@ def is_dict_like(value):
     return hasattr(value, '__getitem__') and hasattr(value, 'keys')
 
 
+def combine_pos_and_kw_args(pos_kwargs, kw_kwargs, func_name):
+    if pos_kwargs is not None:
+        if not is_dict_like(pos_kwargs):
+            raise ValueError('the first argument to .%s must be a dictionary'
+                             % func_name)
+        if kw_kwargs:
+            raise ValueError('cannot specify both keyword and positional '
+                             'arguments to .%s' % func_name)
+        return pos_kwargs
+    else:
+        return kw_kwargs
+
+
 def is_scalar(value):
     """np.isscalar only work on primitive numeric types and (bizarrely)
     excludes 0-d ndarrays; this version does more comprehensive checks
