@@ -1,3 +1,4 @@
+from datetime import timedelta
 import functools
 
 import numpy as np
@@ -49,50 +50,79 @@ def as_variable(obj, key=None, strict=True):
     return obj
 
 
-def _as_compatible_data(data):
+def _maybe_wrap_data(data):
+    """
+    Put pandas.Index and numpy.ndarray arguments in adapter objects to ensure
+    they can be indexed properly.
+
+    NumpyArrayAdapter, PandasIndexAdapter and LazilyIndexedArray should
+    all pass through unmodified.
+    """
+    if isinstance(data, pd.Index):
+        # check pd.Index first since it may be an ndarray subclass
+        return PandasIndexAdapter(data)
+    if isinstance(data, np.ndarray):
+        return NumpyArrayAdapter(data)
+    return data
+
+
+def _as_compatible_data(data, fastpath=False):
     """Prepare and wrap data to put in a Variable.
 
-    Prepare the data:
     - If data does not have the necessary attributes, convert it to ndarray.
     - If data has dtype=datetime64, ensure that it has ns precision. If it's a
       pandas.Timestamp, convert it to datetime64.
     - If data is already a pandas or xray object (other than an Index), just
       use the values.
 
-    Wrap it up:
-    - Finally, put pandas.Index and numpy.ndarray arguments in adapter objects
-      to ensure they can be indexed properly.
-    - NumpyArrayAdapter, PandasIndexAdapter and LazilyIndexedArray should
-      all pass through unmodified.
+    Finally, wrap it up with an adapter if necessary.
     """
-    if isinstance(data, pd.MultiIndex):
-        raise NotImplementedError(
-            'no support yet for using a pandas.MultiIndex in an '
-            'xray.Coordinate')
+    if fastpath and getattr(data, 'ndim', 0) > 0:
+        # can't use fastpath (yet) for scalars
+        return _maybe_wrap_data(data)
+
+    if isinstance(data, pd.Index):
+        if isinstance(data, pd.MultiIndex):
+            raise NotImplementedError(
+                'no support yet for using a pandas.MultiIndex in an '
+                'xray.Coordinate')
+        return _maybe_wrap_data(data)
+
+    if isinstance(data, pd.Timestamp):
+        # TODO: convert, handle datetime objects, too
+        data = np.datetime64(data.value, 'ns')
+    if isinstance(data, timedelta):
+        data = np.timedelta64(getattr(data, 'value', data), 'ns')
 
     # don't check for __len__ or __iter__ so as not to cast if data is a numpy
     # numeric type like np.float32
     required = ['dtype', 'shape', 'size', 'ndim']
-    if isinstance(data, (np.datetime64, pd.Timestamp)):
-        # note: np.datetime64 is ndarray-like
-        data = np.datetime64(data, 'ns')
-    elif (any(not hasattr(data, attr) for attr in required)
-            or isinstance(data, np.string_)):
+    if (any(not hasattr(data, attr) for attr in required)
+            or isinstance(data, (np.string_, np.datetime64, np.timedelta64))):
         # data must be ndarray-like
         data = np.asarray(data)
-    elif not isinstance(data, pd.Index):
-        # we don't want nested self-described arrays
-        data = getattr(data, 'values', data)
 
-    if isinstance(data, pd.Index):
-        # check pd.Index first since it's (currently) an ndarray subclass
-        data = PandasIndexAdapter(data)
-    elif isinstance(data, np.ndarray):
+    # we don't want nested self-described arrays
+    data = getattr(data, 'values', data)
+
+    if isinstance(data, np.ma.MaskedArray):
+        mask = np.ma.getmaskarray(data)
+        if mask.any():
+            dtype, fill_value = common._maybe_promote(data.dtype)
+            data = np.asarray(data, dtype=dtype)
+            data[mask] = fill_value
+        else:
+            data = np.asarray(data)
+
+    if isinstance(data, np.ndarray):
+        data = common._possibly_convert_objects(data)
         if data.dtype.kind == 'M':
+            # TODO: automatically cast arrays of datetime objects as well
             data = np.asarray(data, 'datetime64[ns]')
-        data = NumpyArrayAdapter(data)
+        if data.dtype.kind == 'm':
+            data = np.asarray(data, 'timedelta64[ns]')
 
-    return data
+    return _maybe_wrap_data(data)
 
 
 class NumpyArrayAdapter(utils.NDArrayMixin):
@@ -157,17 +187,12 @@ class PandasIndexAdapter(utils.NDArrayMixin):
                 # pd.Timestamp rather np.than datetime64 but this is easier
                 # (for now)
                 value = np.datetime64('NaT', 'ns')
+            elif isinstance(value, timedelta):
+                value = np.timedelta64(getattr(value, 'value', value), 'ns')
             else:
                 value = np.asarray(value, dtype=self.dtype)
         else:
-            arr = self.array[key]
-            if arr.dtype != self.array.dtype:
-                # pandas<0.14 does dtype inference when slicing:
-                # https://github.com/pydata/pandas/issues/6370
-                # To avoid this, slice values instead if necessary and accept
-                # that we will need to rebuild the index:
-                arr = self.array.values[key]
-            value = PandasIndexAdapter(arr, dtype=self.dtype)
+            value = PandasIndexAdapter(self.array[key], dtype=self.dtype)
 
         return value
 
@@ -192,6 +217,8 @@ def _as_array_or_item(data):
             # convert to a np.datetime64 object, because 0-dimensional ndarrays
             # with dtype=datetime64 are broken :(
             data = np.datetime64(data, 'ns')
+        elif data.dtype.kind == 'm':
+            data = np.timedelta64(data, 'ns')
     return data
 
 
@@ -215,7 +242,7 @@ class Variable(common.AbstractArray):
     form of a Dataset or DataArray should almost always be preferred, because
     they can use more complete metadata in context of coordinate labels.
     """
-    def __init__(self, dims, data, attrs=None, encoding=None):
+    def __init__(self, dims, data, attrs=None, encoding=None, fastpath=False):
         """
         Parameters
         ----------
@@ -235,7 +262,7 @@ class Variable(common.AbstractArray):
             Well behaviored code to serialize a Variable should ignore
             unrecognized encoding items.
         """
-        self._data = _as_compatible_data(data)
+        self._data = _as_compatible_data(data, fastpath=fastpath)
         self._dims = self._parse_dimensions(dims)
         self._attrs = None
         self._encoding = None
@@ -307,8 +334,8 @@ class Variable(common.AbstractArray):
 
     def to_coord(self):
         """Return this variable as an xray.Coordinate"""
-        return Coordinate(self.dims, self._data, self.attrs,
-                          encoding=self.encoding)
+        return Coordinate(self.dims, self._data, self._attrs,
+                          encoding=self._encoding, fastpath=True)
 
     @property
     def as_index(self):
@@ -344,6 +371,12 @@ class Variable(common.AbstractArray):
     def dims(self, value):
         self._dims = self._parse_dimensions(value)
 
+    def _item_key_to_tuple(self, key):
+        if utils.is_dict_like(key):
+            return tuple(key.get(dim, slice(None)) for dim in self.dims)
+        else:
+            return key
+
     def __getitem__(self, key):
         """Return a new Array object whose contents are consistent with
         getting the provided key from the underlying data.
@@ -361,16 +394,17 @@ class Variable(common.AbstractArray):
         If you really want to do indexing like `x[x > 0]`, manipulate the numpy
         array `x.values` directly.
         """
+        key = self._item_key_to_tuple(key)
         key = indexing.expanded_indexer(key, self.ndim)
-        dims = [dim for k, dim in zip(key, self.dims)
-                      if not isinstance(k, (int, np.integer))]
+        dims = tuple(dim for k, dim in zip(key, self.dims)
+                     if not isinstance(k, (int, np.integer)))
         values = self._data[key]
         # orthogonal indexing should ensure the dimensionality is consistent
         if hasattr(values, 'ndim'):
             assert values.ndim == len(dims), (values.ndim, len(dims))
         else:
             assert len(dims) == 0, len(dims)
-        return type(self)(dims, values, self.attrs)
+        return type(self)(dims, values, self._attrs, fastpath=True)
 
     def __setitem__(self, key, value):
         """__setitem__ is overloaded to access the underlying numpy values with
@@ -378,6 +412,7 @@ class Variable(common.AbstractArray):
 
         See __getitem__ for more details.
         """
+        key = self._item_key_to_tuple(key)
         self._data_cached()[key] = value
 
     @property
@@ -424,7 +459,8 @@ class Variable(common.AbstractArray):
         # note:
         # dims is already an immutable tuple
         # attributes and encoding will be copied when the new Array is created
-        return type(self)(self.dims, data, self.attrs, self.encoding)
+        return type(self)(self.dims, data, self._attrs, self._encoding,
+                          fastpath=True)
 
     def __copy__(self):
         return self.copy(deep=False)
@@ -494,7 +530,7 @@ class Variable(common.AbstractArray):
             dims = self.dims[::-1]
         axes = self.get_axis_num(dims)
         data = self.values.transpose(axes)
-        return type(self)(dims, data, self.attrs, self.encoding)
+        return type(self)(dims, data, self._attrs, self._encoding, fastpath=True)
 
     def squeeze(self, dim=None):
         """Return a new Variable object with squeezed data.
@@ -524,6 +560,41 @@ class Variable(common.AbstractArray):
         dims = dict(zip(self.dims, self.shape))
         return common.squeeze(self, dims, dim)
 
+    def set_dims(self, dims):
+        """Return a new variable with expanded dimensions.
+
+        When possible, this operation does not copy this variable's data.
+
+        Parameters
+        ----------
+        dims : str or sequence of str or dict
+            Dimensions to include on the new variable. If a dict, values are
+            used to provide the sizes of new dimensions; otherwise, new
+            dimensions are inserted with length 1.
+
+        Returns
+        -------
+        Variable
+        """
+        if isinstance(dims, basestring):
+            dims = [dims]
+
+        if not utils.is_dict_like(dims):
+            dims_map = dict(zip(self.dims, self.shape))
+            dims = OrderedDict((d, dims_map.get(d, 1)) for d in dims)
+
+        missing_dims = set(self.dims) - set(dims)
+        if missing_dims:
+            raise ValueError('new dimensions must be a superset of existing '
+                             'dimensions')
+
+        self_dims = set(self.dims)
+        exp_dims = tuple(d for d in dims if d not in self_dims) + self.dims
+        exp_data = utils.as_shape(self, [dims[d] for d in exp_dims])
+        expanded_var = Variable(exp_dims, exp_data, self._attrs,
+                                self._encoding, fastpath=True)
+        return expanded_var.transpose(*dims)
+
     def reduce(self, func, dim=None, axis=None, keep_attrs=False,
                **kwargs):
         """Reduce this array by applying `func` along some dimension(s).
@@ -570,7 +641,7 @@ class Variable(common.AbstractArray):
         dims = [dim for n, dim in enumerate(self.dims)
                 if n not in removed_axes]
 
-        attrs = self.attrs if keep_attrs else None
+        attrs = self._attrs if keep_attrs else None
 
         return Variable(dims, data, attrs=attrs)
 
@@ -763,18 +834,20 @@ class Coordinate(Variable):
     """
     _cache_data_class = PandasIndexAdapter
 
-    def __init__(self, name, data, attrs=None, encoding=None):
-        super(Coordinate, self).__init__(name, data, attrs, encoding)
+    def __init__(self, name, data, attrs=None, encoding=None, fastpath=False):
+        super(Coordinate, self).__init__(name, data, attrs, encoding, fastpath)
         if self.ndim != 1:
             raise ValueError('%s objects must be 1-dimensional' %
                              type(self).__name__)
 
     def __getitem__(self, key):
+        key = self._item_key_to_tuple(key)
         values = self._data[key]
         if not hasattr(values, 'ndim') or values.ndim == 0:
-            return Variable((), values, self.attrs, self.encoding)
+            return Variable((), values, self._attrs, self._encoding)
         else:
-            return type(self)(self.dims, values, self.attrs, self.encoding)
+            return type(self)(self.dims, values, self._attrs, self._encoding,
+                              fastpath=True)
 
     def __setitem__(self, key, value):
         raise TypeError('%s values cannot be modified' % type(self).__name__)
@@ -788,7 +861,8 @@ class Coordinate(Variable):
         # there is no need to copy the index values here even if deep=True
         # since pandas.Index objects are immutable
         data = PandasIndexAdapter(self) if deep else self._data
-        return type(self)(self.dims, data, self.attrs, self.encoding)
+        return type(self)(self.dims, data, self._attrs, self._encoding,
+                          fastpath=True)
 
     def _data_equals(self, other):
         return self.to_index().equals(other.to_index())
@@ -858,15 +932,7 @@ def _broadcast_compatible_variables(*variables):
                 raise ValueError('operands cannot be broadcast together '
                                  'with mismatched lengths for dimension %r: %s'
                                  % (d, (all_dims[d], s)))
-
-    broadcast_vars = []
-    for var in variables:
-        data = var.values[(Ellipsis,) + (None,) * (len(all_dims) - var.ndim)]
-        var_dims = set(var.dims)
-        dims = var.dims + tuple(d for d in all_dims if d not in var_dims)
-        expanded_var = Variable(dims, data, var.attrs, var.encoding)
-        broadcast_vars.append(expanded_var.transpose(*all_dims))
-    return tuple(broadcast_vars)
+    return tuple(var.set_dims(all_dims) for var in variables)
 
 
 def _set_data_directly(var, values):
