@@ -1,11 +1,7 @@
-from xray.conventions import cf_decoder
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
 from io import BytesIO
 import contextlib
 import os.path
+import pickle
 import tempfile
 import unittest
 import sys
@@ -13,7 +9,7 @@ import sys
 import numpy as np
 import pandas as pd
 
-from xray import Dataset, open_dataset, backends
+from xray import Dataset, open_dataset, backends, decode_cf
 from xray.core.pycompat import iteritems, PY3
 
 from . import TestCase, requires_scipy, requires_netCDF4, requires_pydap
@@ -60,23 +56,19 @@ class DatasetIOTestCases(object):
             raise unittest.SkipTest('scipy.io.netcdf is broken on Python 3')
 
         expected = create_test_data()
-        expected['xray_awesomeness'] = ([], np.array(1.e9),
-                                        {'units': 'units of awesome'})
-        with self.create_store() as store:
-            expected.dump_to_store(store)
-            # the test data contains times.  In case the store
-            # cf_encodes them we need to cf_decode them.
-            actual = Dataset.load_store(store, cf_decoder)
-        self.assertDatasetAllClose(expected, actual)
+        expected['float_var'] = ([], 1.0e9, {'units': 'units of awesome'})
+        expected['string_var'] = ([], np.array('foobar', dtype='S'))
+        with self.roundtrip(expected) as actual:
+            self.assertDatasetAllClose(expected, actual)
 
     def test_write_store(self):
         expected = create_test_data()
         with self.create_store() as store:
             expected.dump_to_store(store)
-            # the test data contains times.  In case the store
-            # cf_encodes them we need to cf_decode them.
-            actual = Dataset.load_store(store, cf_decoder)
-        self.assertDatasetAllClose(expected, actual)
+            # we need to cf decode the store because it has time and
+            # non-dimension coordinates
+            actual = decode_cf(store)
+            self.assertDatasetAllClose(expected, actual)
 
     def test_roundtrip_test_data(self):
         expected = create_test_data()
@@ -152,6 +144,18 @@ class DatasetIOTestCases(object):
                 expected['x'] = expected['x'].astype('S')
             self.assertDatasetIdentical(expected, actual)
 
+    def test_roundtrip_datetime_data(self):
+        times = pd.to_datetime(['2000-01-01', '2000-01-02', 'NaT'])
+        expected = Dataset({'t': ('t', times)})
+        with self.roundtrip(expected) as actual:
+            self.assertDatasetIdentical(expected, actual)
+
+    def test_roundtrip_timedelta_data(self):
+        time_deltas = pd.to_timedelta(['1h', '2h', 'NaT'])
+        expected = Dataset({'td': ('td', time_deltas)})
+        with self.roundtrip(expected) as actual:
+            self.assertDatasetIdentical(expected, actual)
+
     def test_roundtrip_example_1_netcdf(self):
         expected = open_example_dataset('example_1.nc')
         with self.roundtrip(expected) as actual:
@@ -171,6 +175,29 @@ class DatasetIOTestCases(object):
                 with open_example_dataset('example_1.nc') as actual:
                     self.assertDatasetIdentical(expected, actual)
 
+    def test_roundtrip_coordinates(self):
+        original = Dataset({'foo': ('x', [0, 1])},
+                           {'x': [2, 3], 'y': ('a', [42]), 'z': ('x', [4, 5])})
+
+        with self.roundtrip(original) as actual:
+            self.assertDatasetIdentical(original, actual)
+
+        expected = original.drop_vars('foo')
+        with self.roundtrip(expected) as actual:
+            self.assertDatasetIdentical(expected, actual)
+
+        expected = original.copy()
+        expected.attrs['coordinates'] = 'something random'
+        with self.assertRaisesRegexp(ValueError, 'cannot serialize'):
+            with self.roundtrip(expected):
+                pass
+
+        expected = original.copy(deep=True)
+        expected['foo'].attrs['coordinates'] = 'something random'
+        with self.assertRaisesRegexp(ValueError, 'cannot serialize'):
+            with self.roundtrip(expected):
+                pass
+
     def test_orthogonal_indexing(self):
         in_memory = create_test_data()
         with self.roundtrip(in_memory) as on_disk:
@@ -359,7 +386,7 @@ class NetCDF4DataTest(CFEncodedDataTest, TestCase):
                 self.assertEqual(v, actual['var2'].encoding[k])
 
         # regression test for #156
-        expected = data.isel(dim1=0).reset_coords()
+        expected = data.isel(dim1=0)
         with self.roundtrip(expected) as actual:
             self.assertDatasetEqual(expected, actual)
 
@@ -483,6 +510,33 @@ class NetCDF4DataTest(CFEncodedDataTest, TestCase):
                     self.assertEqual(ds.variables['time'].getncattr('units'), units)
                     self.assertArrayEqual(ds.variables['time'], np.arange(10) + 4)
 
+    def test_coordinates_encoding(self):
+        def equals_latlon(obj):
+            return obj == 'lat lon' or obj == 'lon lat'
+
+        original = Dataset({'temp': ('x', [0, 1]), 'precip': ('x', [0, -1])},
+                           {'lat': ('x', [2, 3]), 'lon': ('x', [4, 5])})
+        with self.roundtrip(original) as actual:
+            self.assertDatasetIdentical(actual, original)
+        with create_tmp_file() as tmp_file:
+            original.to_netcdf(tmp_file)
+            with open_dataset(tmp_file, decode_coords=False) as ds:
+                self.assertTrue(equals_latlon(ds['temp'].attrs['coordinates']))
+                self.assertTrue(equals_latlon(ds['precip'].attrs['coordinates']))
+                self.assertNotIn('coordinates', ds.attrs)
+                self.assertNotIn('coordinates', ds['lat'].attrs)
+                self.assertNotIn('coordinates', ds['lon'].attrs)
+
+        modified = original.drop_vars('temp', 'precip')
+        with self.roundtrip(modified) as actual:
+            self.assertDatasetIdentical(actual, modified)
+        with create_tmp_file() as tmp_file:
+            modified.to_netcdf(tmp_file)
+            with open_dataset(tmp_file, decode_coords=False) as ds:
+                self.assertTrue(equals_latlon(ds.attrs['coordinates']))
+                self.assertNotIn('coordinates', ds['lat'].attrs)
+                self.assertNotIn('coordinates', ds['lon'].attrs)
+
 
 @requires_netCDF4
 @requires_scipy
