@@ -282,6 +282,11 @@ class TestDataArray(TestCase):
                   I[x.values > 3], I[x.variable > 3], I[x > 3], I[x > 3, y > 3]]:
             self.assertVariableEqual(self.v[i], self.dv[i])
 
+    def test_getitem_dict(self):
+        actual = self.dv[{'x': slice(3), 'y': 0}]
+        expected = self.dv.isel(x=slice(3), y=0)
+        self.assertDataArrayIdentical(expected, actual)
+
     def test_getitem_coords(self):
         orig = DataArray([[10], [20]],
                          {'x': [1, 2], 'y': [3], 'z': 4,
@@ -316,11 +321,11 @@ class TestDataArray(TestCase):
         self.assertDataArrayIdentical(expected, actual)
 
     def test_isel(self):
-        self.assertDatasetIdentical(self.dv[0].to_dataset(), self.ds.isel(x=0))
-        self.assertDatasetIdentical(self.dv[:3, :5].to_dataset(),
-                                    self.ds.isel(x=slice(3), y=slice(5)))
+        self.assertDataArrayIdentical(self.dv[0], self.dv.isel(x=0))
         self.assertDataArrayIdentical(self.dv, self.dv.isel(x=slice(None)))
         self.assertDataArrayIdentical(self.dv[:3], self.dv.isel(x=slice(3)))
+        self.assertDataArrayIdentical(self.dv[:3, :5],
+                                      self.dv.isel(x=slice(3), y=slice(5)))
 
     def test_sel(self):
         self.ds['x'] = ('x', np.array(list('abcdefghij')))
@@ -340,12 +345,15 @@ class TestDataArray(TestCase):
         da = self.ds['foo']
         self.assertDataArrayIdentical(da[:3], da.loc[:'c'])
         self.assertDataArrayIdentical(da[1], da.loc['b'])
+        self.assertDataArrayIdentical(da[1], da.loc[{'x': 'b'}])
         self.assertDataArrayIdentical(da[:3], da.loc[['a', 'b', 'c']])
         self.assertDataArrayIdentical(da[:3, :4],
                                       da.loc[['a', 'b', 'c'], np.arange(4)])
         self.assertDataArrayIdentical(da[:, :4], da.loc[:, self.ds['y'] < 4])
         da.loc['a':'j'] = 0
         self.assertTrue(np.all(da.values == 0))
+        da.loc[{'x': slice('a', 'j')}] = 2
+        self.assertTrue(np.all(da.values == 2))
 
     def test_loc_single_boolean(self):
         data = DataArray([0, 1], coords=[[True, False]])
@@ -462,6 +470,12 @@ class TestDataArray(TestCase):
         expected[:2, :2] = bar
         self.assertDataArrayIdentical(bar.reindex_like(foo), expected)
 
+        # regression test for #279
+        expected = DataArray(np.random.randn(5), dims=["time"])
+        time2 = DataArray(np.arange(5), dims="time2")
+        actual = expected.reindex(time=time2)
+        self.assertDataArrayIdentical(actual, expected)
+
     def test_rename(self):
         renamed = self.dv.rename('bar')
         self.assertDatasetIdentical(
@@ -893,6 +907,15 @@ class TestDataArray(TestCase):
         self.assertDataArrayIdentical(dv1, self.dv[:5])
         self.assertDataArrayIdentical(dv2, self.dv[:5])
 
+    def test_align_dtype(self):
+        # regression test for #264
+        x1 = np.arange(30)
+        x2 = np.arange(5, 35)
+        a = DataArray(np.random.random((30,)).astype('f32'), {'x': x1})
+        b = DataArray(np.random.random((30,)).astype('f32'), {'x': x2})
+        c, d = align(a, b, join='outer')
+        self.assertEqual(c.dtype, np.float32)
+
     def test_broadcast_arrays(self):
         x = DataArray([1, 2], coords=[('a', [-1, -2])], name='x')
         y = DataArray([1, 2], coords=[('b', [3, 4])], name='y')
@@ -972,6 +995,34 @@ class TestDataArray(TestCase):
         self.assertDataArrayIdentical(expected_da,
                                       DataArray.from_series(actual))
 
+    def test_to_and_from_cdms2(self):
+        try:
+            import cdms2
+        except ImportError:
+            raise unittest.SkipTest('cdms2 not installed')
+
+        original = DataArray(np.arange(6).reshape(2, 3),
+                             [('distance', [-2, 2], {'units': 'meters'}),
+                              ('time', pd.date_range('2000-01-01', periods=3))],
+                             name='foo', attrs={'baz': 123})
+        expected_coords = [Coordinate('distance', [-2, 2]),
+                           Coordinate('time', [0, 1, 2])]
+        actual = original.to_cdms2()
+        self.assertArrayEqual(actual, original)
+        self.assertEqual(actual.id, original.name)
+        self.assertItemsEqual(actual.getAxisIds(), original.dims)
+        for axis, coord in zip(actual.getAxisList(), expected_coords):
+            self.assertEqual(axis.id, coord.name)
+            self.assertArrayEqual(axis, coord.values)
+        self.assertEqual(actual.baz, original.attrs['baz'])
+
+        component_times = actual.getAxis(1).asComponentTime()
+        self.assertEqual(len(component_times), 3)
+        self.assertEqual(str(component_times[0]), '2000-1-1 0:0:0.0')
+
+        roundtripped = DataArray.from_cdms2(actual)
+        self.assertDataArrayIdentical(original, roundtripped)
+
     def test_to_dataset(self):
         unnamed = DataArray([1, 2], dims='x')
         actual = unnamed.to_dataset()
