@@ -175,6 +175,23 @@ class TestDataset(TestCase):
         self.assertNotIn('var1', ds.coords)
         self.assertEqual(len(ds.coords), 5)
 
+    def test_attr_access(self):
+        ds = Dataset({'tmin': ('x', [42], {'units': 'Celcius'})},
+                     attrs={'title': 'My test data'})
+        self.assertDataArrayIdentical(ds.tmin, ds['tmin'])
+        self.assertDataArrayIdentical(ds.tmin.x, ds.x)
+
+        self.assertEqual(ds.title, ds.attrs['title'])
+        self.assertEqual(ds.tmin.units, ds['tmin'].attrs['units'])
+
+        self.assertLessEqual(set(['tmin', 'title']), set(dir(ds)))
+        self.assertIn('units', set(dir(ds.tmin)))
+
+        # should defer to variable of same name
+        ds.attrs['tmin'] = -999
+        self.assertEqual(ds.attrs['tmin'], -999)
+        self.assertDataArrayIdentical(ds.tmin, ds['tmin'])
+
     def test_variable(self):
         a = Dataset()
         d = np.random.random((10, 3))
@@ -449,7 +466,7 @@ class TestDataset(TestCase):
                                 data.sel(time='2000-01-01'))
         self.assertDatasetEqual(data.isel(time=slice(10)),
                                 data.sel(time=slice('2000-01-01',
-                                                   '2000-01-10')))
+                                                    '2000-01-10')))
         self.assertDatasetEqual(data, data.sel(time=slice('1999', '2005')))
         times = pd.date_range('2000-01-01', periods=3)
         self.assertDatasetEqual(data.isel(time=slice(3)),
@@ -457,6 +474,25 @@ class TestDataset(TestCase):
         self.assertDatasetEqual(data.isel(time=slice(3)),
                                 data.sel(time=(data['time.dayofyear'] <= 3)))
 
+        td = pd.to_timedelta(np.arange(3), unit='days')
+        data = Dataset({'x': ('td', np.arange(3)), 'td': td})
+        self.assertDatasetEqual(data, data.sel(td=td))
+        self.assertDatasetEqual(data, data.sel(td=slice('3 days')))
+        self.assertDatasetEqual(data.isel(td=0), data.sel(td='0 days'))
+        self.assertDatasetEqual(data.isel(td=0), data.sel(td='0h'))
+        self.assertDatasetEqual(data.isel(td=slice(1, 3)),
+                                data.sel(td=slice('1 days', '2 days')))
+
+    def test_loc(self):
+        data = create_test_data()
+        expected = data.sel(dim3='a')
+        actual = data.loc[dict(dim3='a')]
+        self.assertDatasetIdentical(expected, actual)
+        with self.assertRaisesRegexp(TypeError, 'can only lookup dict'):
+            data.loc['a']
+        with self.assertRaises(TypeError):
+            data.loc[dict(dim3='a')] = 0
+
     def test_reindex_like(self):
         data = create_test_data()
         data['letters'] = ('dim3', 10 * ['a'])
@@ -491,6 +527,25 @@ class TestDataset(TestCase):
         actual = data.reindex(dim1=data['dim1'][:10].to_index())
         self.assertDatasetIdentical(actual, expected)
 
+        # test dict-like argument
+        actual = data.reindex({'dim1': data['dim1'][:10]})
+        self.assertDatasetIdentical(actual, expected)
+        with self.assertRaisesRegexp(ValueError, 'cannot specify both'):
+            data.reindex({'x': 0}, x=0)
+        with self.assertRaisesRegexp(ValueError, 'dictionary'):
+            data.reindex('foo')
+
+        # out of order
+        expected = data.sel(dim1=data['dim1'][:10:-1])
+        actual = data.reindex(dim1=data['dim1'][:10:-1])
+        self.assertDatasetIdentical(actual, expected)
+
+        # regression test for #279
+        expected = Dataset({'x': ('time', np.random.randn(5))})
+        time2 = DataArray(np.arange(5), dims="time2")
+        actual = expected.reindex(time=time2)
+        self.assertDatasetIdentical(actual, expected)
+
     def test_align(self):
         left = create_test_data()
         right = left.copy(deep=True)
@@ -686,6 +741,10 @@ class TestDataset(TestCase):
                              name='numbers')
         self.assertDataArrayIdentical(expected, actual)
 
+        actual = data[dict(dim1=0)]
+        expected = data.isel(dim1=0)
+        self.assertDatasetIdentical(expected, actual)
+
     def test_virtual_variables(self):
         # access virtual variables
         data = create_test_data()
@@ -744,6 +803,9 @@ class TestDataset(TestCase):
         data1['A'] = 3 * data2['A']
         self.assertVariableEqual(data1['A'], 3 * data2['A'])
 
+        with self.assertRaises(NotImplementedError):
+            data1[{'x': 0}] = 0
+
     def test_delitem(self):
         data = create_test_data()
         all_items = set(data)
@@ -1010,6 +1072,20 @@ class TestDataset(TestCase):
         expected = Dataset()
         self.assertDatasetIdentical(expected, actual)
 
+        # regression test for GH278
+        ds = Dataset({'x': pd.Index(['bar']), 'a': ('y', [1])}).isel(x=0)
+        # use .loc to ensure consistent results on Python 3
+        actual = ds.to_dataframe().loc[:, ['a', 'x']]
+        expected = pd.DataFrame([[1, 'bar']], index=pd.Index([0], name='y'),
+                                columns=['a', 'x'])
+        assert expected.equals(actual), (expected, actual)
+
+        ds = Dataset({'x': [0], 'y': [1]})
+        actual = ds.to_dataframe()
+        idx = pd.MultiIndex.from_arrays([[0], [1]], names=['x', 'y'])
+        expected = pd.DataFrame([[]], index=idx)
+        assert expected.equals(actual), (expected, actual)
+
     def test_pickle(self):
         data = create_test_data()
         roundtripped = pickle.loads(pickle.dumps(data))
