@@ -1,6 +1,6 @@
 from collections import namedtuple
 from copy import copy, deepcopy
-from datetime import datetime
+from datetime import datetime, timedelta
 from textwrap import dedent
 
 import numpy as np
@@ -38,6 +38,12 @@ class VariableSubclassTestCases(object):
         v.attrs['foo'] = 'baz'
         self.assertEqual(v.attrs['foo'], 'baz')
 
+    def test_getitem_dict(self):
+        v = self.cls(['x'], np.random.randn(5))
+        actual = v[{'x': 0}]
+        expected = v[0]
+        self.assertVariableIdentical(expected, actual)
+
     def assertIndexedLikeNDArray(self, variable, expected_value0,
                                  expected_dtype=None):
         """Given a 1-dimensional variable, verify that the variable is indexed
@@ -82,7 +88,7 @@ class VariableSubclassTestCases(object):
     def test_index_0d_datetime(self):
         d = datetime(2000, 1, 1)
         x = self.cls(['x'], [d])
-        self.assertIndexedLikeNDArray(x, d)
+        self.assertIndexedLikeNDArray(x, np.datetime64(d))
 
         x = self.cls(['x'], [np.datetime64(d)])
         self.assertIndexedLikeNDArray(x, np.datetime64(d), 'datetime64[ns]')
@@ -90,6 +96,15 @@ class VariableSubclassTestCases(object):
         x = self.cls(['x'], pd.DatetimeIndex([d]))
         self.assertIndexedLikeNDArray(x, np.datetime64(d), 'datetime64[ns]')
 
+    def test_index_0d_timedelta64(self):
+        td = timedelta(hours=1)
+
+        x = self.cls(['x'], [np.timedelta64(td)])
+        self.assertIndexedLikeNDArray(x, np.timedelta64(td), 'timedelta64[ns]')
+
+        x = self.cls(['x'], pd.to_timedelta([td]))
+        self.assertIndexedLikeNDArray(x, np.timedelta64(td), 'timedelta64[ns]')
+
     def test_index_0d_not_a_time(self):
         d = np.datetime64('NaT')
         x = self.cls(['x'], [d])
@@ -133,6 +148,42 @@ class VariableSubclassTestCases(object):
         expected = np.datetime64('2000-01-01T00Z', 'ns')
         self.assertEqual(x[0].values, expected)
 
+    def test_datetime64_conversion(self):
+        times = pd.date_range('2000-01-01', periods=3)
+        for values, preserve_source in [
+                (times, False),
+                (times.values, True),
+                (times.values.astype('datetime64[s]'), False),
+                (times.to_pydatetime(), False),
+               ]:
+            v = self.cls(['t'], values)
+            self.assertEqual(v.dtype, np.dtype('datetime64[ns]'))
+            self.assertArrayEqual(v.values, times.values)
+            self.assertEqual(v.values.dtype, np.dtype('datetime64[ns]'))
+            same_source = source_ndarray(v.values) is source_ndarray(values)
+            if preserve_source and self.cls is Variable:
+                self.assertTrue(same_source)
+            else:
+                self.assertFalse(same_source)
+
+    def test_timedelta64_conversion(self):
+        times = pd.timedelta_range(start=0, periods=3)
+        for values, preserve_source in [
+                (times, False),
+                (times.values, True),
+                (times.values.astype('timedelta64[s]'), False),
+                (times.to_pytimedelta(), False),
+               ]:
+            v = self.cls(['t'], values)
+            self.assertEqual(v.dtype, np.dtype('timedelta64[ns]'))
+            self.assertArrayEqual(v.values, times.values)
+            self.assertEqual(v.values.dtype, np.dtype('timedelta64[ns]'))
+            same_source = source_ndarray(v.values) is source_ndarray(values)
+            if preserve_source and self.cls is Variable:
+                self.assertTrue(same_source)
+            else:
+                self.assertFalse(same_source)
+
     def test_pandas_data(self):
         v = self.cls(['x'], pd.Series([0, 1, 2], index=[3, 2, 1]))
         self.assertVariableIdentical(v, v[[0, 1, 2]])
@@ -318,29 +369,29 @@ class TestVariable(TestCase, VariableSubclassTestCases):
         v = Coordinate('x', np.arange(5))
         self.assertEqual(2, v.searchsorted(2))
 
-    def test_datetime64_conversion(self):
-        # verify that datetime64 is always converted to ns precision with
-        # sources preserved
-        values = np.datetime64('2000-01-01T00')
-        v = Variable([], values)
-        self.assertEqual(v.dtype, np.dtype('datetime64[ns]'))
-        self.assertEqual(v.values, values)
-        self.assertEqual(v.values.dtype, np.dtype('datetime64[ns]'))
+    def test_datetime64_conversion_scalar(self):
+        expected = np.datetime64('2000-01-01T00:00:00Z', 'ns')
+        for values in [
+                 np.datetime64('2000-01-01T00Z'),
+                 pd.Timestamp('2000-01-01T00'),
+                 datetime(2000, 1, 1),
+                ]:
+            v = Variable([], values)
+            self.assertEqual(v.dtype, np.dtype('datetime64[ns]'))
+            self.assertEqual(v.values, expected)
+            self.assertEqual(v.values.dtype, np.dtype('datetime64[ns]'))
 
-        values = pd.date_range('2000-01-01', periods=3).values.astype(
-            'datetime64[s]')
-        v = Variable(['t'], values)
-        self.assertEqual(v.dtype, np.dtype('datetime64[ns]'))
-        self.assertArrayEqual(v.values, values)
-        self.assertEqual(v.values.dtype, np.dtype('datetime64[ns]'))
-        self.assertIsNot(source_ndarray(v.values), values)
-
-        values = pd.date_range('2000-01-01', periods=3).values.copy()
-        v = Variable(['t'], values)
-        self.assertEqual(v.dtype, np.dtype('datetime64[ns]'))
-        self.assertArrayEqual(v.values, values)
-        self.assertEqual(v.values.dtype, np.dtype('datetime64[ns]'))
-        self.assertIs(source_ndarray(v.values), values)
+    def test_timedelta64_conversion_scalar(self):
+        expected = np.timedelta64(24 * 60 * 60 * 10 ** 9, 'ns')
+        for values in [
+                 np.timedelta64(1, 'D'),
+                 pd.Timedelta('1 day'),
+                 timedelta(days=1),
+                ]:
+            v = Variable([], values)
+            self.assertEqual(v.dtype, np.dtype('timedelta64[ns]'))
+            self.assertEqual(v.values, expected)
+            self.assertEqual(v.values.dtype, np.dtype('timedelta64[ns]'))
 
     def test_0d_str(self):
         v = Variable([], u'foo')
@@ -356,6 +407,12 @@ class TestVariable(TestCase, VariableSubclassTestCases):
         self.assertEqual(v.dtype, np.dtype('datetime64[ns]'))
         self.assertEqual(v.values, np.datetime64('2000-01-01T00Z', 'ns'))
 
+    def test_0d_timedelta(self):
+        for td in [pd.to_timedelta('1s'), np.timedelta64(1, 's')]:
+            v = Variable([], td)
+            self.assertEqual(v.dtype, np.dtype('timedelta64[ns]'))
+            self.assertEqual(v.values, np.timedelta64(10 ** 9, 'ns'))
+
     def test_equals_and_identical(self):
         d = np.random.rand(10, 3)
         d[0, 0] = np.nan
@@ -508,6 +565,27 @@ class TestVariable(TestCase, VariableSubclassTestCases):
         with self.assertRaisesRegexp(ValueError, 'not found in array dim'):
             v.get_axis_num('foobar')
 
+    def test_set_dims(self):
+        v = Variable(['x'], [0, 1])
+        actual = v.set_dims(['x', 'y'])
+        expected = Variable(['x', 'y'], [[0], [1]])
+        self.assertVariableIdentical(actual, expected)
+
+        actual = v.set_dims(['y', 'x'])
+        self.assertVariableIdentical(actual, expected.T)
+
+        actual = v.set_dims(OrderedDict([('x', 2), ('y', 2)]))
+        expected = Variable(['x', 'y'], [[0, 0], [1, 1]])
+        self.assertVariableIdentical(actual, expected)
+
+        v = Variable(['foo'], [0, 1])
+        actual = v.set_dims('foo')
+        expected = v
+        self.assertVariableIdentical(actual, expected)
+
+        with self.assertRaisesRegexp(ValueError, 'must be a superset'):
+            v.set_dims(['z'])
+
     def test_broadcasting_math(self):
         x = np.random.randn(2, 3)
         v = Variable(['a', 'b'], x)
@@ -634,18 +712,6 @@ class TestCoordinate(TestCase, VariableSubclassTestCases):
         with self.assertRaisesRegexp(TypeError, 'cannot be modified'):
             x[:] = 0
 
-    def test_avoid_index_dtype_inference(self):
-        # verify our work-around for (pandas<0.14):
-        # https://github.com/pydata/pandas/issues/6370
-        data = pd.date_range('2000-01-01', periods=3).to_pydatetime()
-        t = Coordinate('t', data)
-        self.assertArrayEqual(t.values[:2], data[:2])
-        self.assertArrayEqual(t[:2].values, data[:2])
-        self.assertArrayEqual(t.values[:2], data[:2])
-        self.assertArrayEqual(t[:2].values, data[:2])
-        self.assertEqual(t.dtype, object)
-        self.assertEqual(t[:2].dtype, object)
-
     def test_name(self):
         coord = Coordinate('x', [10.0])
         self.assertEqual(coord.name, 'x')
@@ -672,28 +738,42 @@ class TestAsCompatibleData(TestCase):
             self.assertEqual(NumpyArrayAdapter, type(actual))
             self.assertEqual(np.dtype(int), actual.dtype)
 
+    def test_masked_array(self):
+        original = np.ma.MaskedArray(np.arange(5))
+        expected = np.arange(5)
+        actual = _as_compatible_data(original)
+        self.assertArrayEqual(expected, actual)
+        self.assertEqual(np.dtype(int), actual.dtype)
+
+        original = np.ma.MaskedArray(np.arange(5), mask=4 * [False] + [True])
+        expected = np.arange(5.0)
+        expected[-1] = np.nan
+        actual = _as_compatible_data(original)
+        self.assertArrayEqual(expected, actual)
+        self.assertEqual(np.dtype(float), actual.dtype)
+
     def test_datetime(self):
-        expected = np.datetime64('2000-01-01T00')
+        expected = np.datetime64('2000-01-01T00Z')
         actual = _as_compatible_data(expected)
         self.assertEqual(expected, actual)
-        self.assertEqual(np.datetime64, type(actual))
+        self.assertEqual(NumpyArrayAdapter, type(actual))
         self.assertEqual(np.dtype('datetime64[ns]'), actual.dtype)
 
-        expected = np.array([np.datetime64('2000-01-01T00')])
+        expected = np.array([np.datetime64('2000-01-01T00Z')])
         actual = _as_compatible_data(expected)
         self.assertEqual(np.asarray(expected), actual)
         self.assertEqual(NumpyArrayAdapter, type(actual))
         self.assertEqual(np.dtype('datetime64[ns]'), actual.dtype)
 
-        expected = np.array([np.datetime64('2000-01-01T00', 'ns')])
+        expected = np.array([np.datetime64('2000-01-01T00Z', 'ns')])
         actual = _as_compatible_data(expected)
         self.assertEqual(np.asarray(expected), actual)
         self.assertEqual(NumpyArrayAdapter, type(actual))
         self.assertEqual(np.dtype('datetime64[ns]'), actual.dtype)
         self.assertIs(expected, source_ndarray(np.asarray(actual)))
 
-        expected = pd.Timestamp('2000-01-01T00').to_datetime()
-        actual = _as_compatible_data(expected)
+        expected = np.datetime64('2000-01-01T00Z', 'ns')
+        actual = _as_compatible_data(datetime(2000, 1, 1))
         self.assertEqual(np.asarray(expected), actual)
         self.assertEqual(NumpyArrayAdapter, type(actual))
-        self.assertEqual(np.dtype('O'), actual.dtype)
+        self.assertEqual(np.dtype('datetime64[ns]'), actual.dtype)
